from __future__ import absolute_import, division, print_function

from octane import api_resources


OBJECT_CLASSES = {
    # data structures
    api_resources.ListObject.OBJECT_NAME: api_resources.ListObject,
    # business objects
    api_resources.Customer.OBJECT_NAME: api_resources.Customer,
    api_resources.Mapping.OBJECT_NAME: api_resources.Mapping,
    api_resources.PaymentGatewayCredential.OBJECT_NAME: api_resources.PaymentGatewayCredential,
    api_resources.Measurement.OBJECT_NAME: api_resources.Measurement,
    api_resources.Meter.OBJECT_NAME: api_resources.Meter,
    api_resources.PricePlan.OBJECT_NAME: api_resources.PricePlan,
    api_resources.PriceList.OBJECT_NAME: api_resources.PriceList,
    api_resources.Subscription.OBJECT_NAME: api_resources.Subscription,
}



#!/usr/bin/env python

import octane
from os import getenv
from flask import Flask, request, jsonify, make_response, send_from_directory

octane.api_key = getenv("OCTANE_API_KEY")

app = Flask("antler-db-cloud-shop")

# Application settings and defaults
port = getenv("APP_PORT", 3000)
bind = getenv("APP_BIND", "127.0.0.1")
meter_name = getenv("OCTANE_METER_NAME", "hours")
price_plan_name = getenv("OCTANE_PRICE_PLAN_NAME", "standard")
price_plan_rate = getenv("OCTANE_PRICE_PLAN_RATE", 30)


@app.route("/")
@app.route("/index.html", methods=["GET"])
def public_index_html():
    return send_from_directory("public", "index.html")


@app.route("/scripts.js", methods=["GET"])
def public_scripts_js():
    return send_from_directory("public", "scripts.js")


@app.route("/styles.css", methods=["GET"])
def public_styles_css():
    return send_from_directory("public", "styles.css")


@app.route("/api/customers", methods=["GET"])
def api_customers():
    print("[octane] Listing customers in account")
    try:
        customers = octane.Customer.list()
        return jsonify(customers)
    except octane.error.APIError as e:
        print("[octane] Error listing customers in account")
        return e.json_body, e.http_status


@app.route("/api/customers/<name>", methods=["DELETE"])
def api_customers_name_delete(name):
    return {
        "code": 501,
        "status": "Not Implemented",
        "message": "Python version does not yet support customer deletion"
    }, 501

    # TODO: uncomment below once deletion is supported
    """
    print(f"[octane] Attempting to unsubscribe customer \"{name}\" " +
          f"from price plan \"{price_plan_name}\"")
    try:
        octane.Customer.delete_subscription(name)
        print(f"[octane] Customer \"{name}\" successfully unsubscribed " +
              f"from price plan \"{price_plan_name}\"")
        print("f[octane] Attempting to delete customer \"{name}\"")
        try:
            octane.Customer.delete(name)
            return {
                "code": 200,
                "message": "success"
            }
        except octane.error.APIError as e:
            print(f"[octane] Error deleting customer \"{name}\"")
            return e.json_body, e.http_status
    except octane.error.APIError as e:
        print(f"[octane] Error unsubscribing customer \"{name}\" " +
              f"from price plan \"{price_plan_name}\"")
        return e.json_body, e.http_status
    """


@app.route("/api/customers", methods=["POST"])
def api_customers_post():
    data = request.json
    name = data["name"]
    print(f"[octane] Attempting to create new customer \"{name}\"")
    try:
        octane.Customer.create(
            name=name,
            measurement_mappings=[{
                "label": "customer_name",
                "value_regex": name
            }]
        )
        print(f"[octane] Customer \"{name}\" successfully created")
        print(f"[octane] Attempting to subscribe customer \"{name}\" " +
              f"to price plan \"{price_plan_name}\"")
        try:
            octane.Customer.create_subscription(name, price_plan_name=price_plan_name)
            print(f"[octane] Successfully subscribed customer \"{name}\"" +
                  f"to price plan \"{price_plan_name}\"")
            return {
                "code": 201,
                "message": "success"
            }, 201
        except octane.error.APIError as e:
            print(f"[octane] Error subscribing customer \"{name}\" " +
                  f"to price plan \"{price_plan_name}\"")
            return e.json_body, e.http_status
    except octane.error.APIError as e:
        print("[octane] Error creating customer \"{name}\"")
        return e.json_body, e.http_status


@app.route("/api/hours", methods=["POST"])
def api_hours_post():
    data = request.json
    name = data["name"]
    hours = data["hours"]
    print(f"[octane] Attempting to create measurement for customer \"{name}\"")
    try:
        octane.Measurement.create(
            meter_name=meter_name,
            value=int(hours),
            labels={
                "customer_name": name
            }
        )
        print(f"[octane] Measurement for customer \"{name}\" successfully created")
        return {
            "code": 201,
            "message": "success"
        }, 201
    except octane.error.APIError as e:
        print("[octane] Error creating measurement for customer \"{name}\"")
        return e.json_body, e.http_status


def check_octane_api_key():
    if not octane.api_key:
        raise Exception("Must set OCTANE_API_KEY.")


def check_octane_resource_meter():
    print(f"[octane] Checking if meter \"{meter_name}\" exists")
    try:
        octane.Meter.retrieve(meter_name)
        print(f"[octane] Meter \"{meter_name}\" already exists")
    except octane.error.APIError as e:
        if e.http_status == 401:
            raise Exception("Unauthorized, please check your OCTANE_API_KEY.")
        print(f"[octane] Meter \"{meter_name}\" does not exist, creating")
        try:
            octane.Meter.create(
                name=meter_name,
                display_name="Number of hours worked",
                meter_type="COUNTER",
                unit_name="hour",
                is_incremental=True,
                expected_labels=["customer_name"]
            )
            print(f"[octane] Meter \"{meter_name}\" successfully created")
        except octane.error.APIError as e:
            print(e.http_body)
            raise Exception("Unable to create meter")


def check_octane_resource_price_plan():
    print(f"[octane] Checking if price plan \"{price_plan_name}\" exists")
    try:
        octane.PricePlan.retrieve(price_plan_name)
        print(f"[octane] Price plan \"{price_plan_name}\" already exists")
    except octane.error.APIError as e:
        if e.http_status == 401:
            raise Exception("Unauthorized, please check your OCTANE_API_KEY.")
        print(f"[octane] Price plan \"{price_plan_name}\" does not exist, creating")
        try:
            rate = int(price_plan_rate) * 100  # convert dollars to cents
            octane.PricePlan.create(
                name=price_plan_name,
                period="month",
                metered_components=[{
                    "meter_name": meter_name,
                    "price_scheme": {
                        "prices": [{
                            "price": rate
                        }],
                        "scheme_type": "FLAT",
                        "unit_name": "hour"
                    }
                }]
            )
            print(f"[octane] Price plan \"{price_plan_name}\" successfully created")
        except octane.error.APIError as e:
            print(e.http_body)
            raise Exception("Unable to create price plan")


if __name__ == "__main__":
    check_octane_api_key()
    check_octane_resource_meter()
    check_octane_resource_price_plan()
    print(f"[server] Listening at http://{bind}:{port}/")
    app.run(host=bind, port=port)



from datetime import datetime, timezone, timedelta
import dateutil.parser
import time

import octane

octane.api_base = "http://localhost:8080"
octane.api_key = (
    "9aed8aec1b625359132245af12e8108f71ccb9b58b705f18845d9a5fd89cc362"
)

NUM_CUSTOMERS = 100
NUM_SOURCES = 10


def customer_name(i):
    return f"customer_{i}"


for i in range(NUM_CUSTOMERS):
    name = customer_name(i)
    octane.Customer.create(name=name, display_name=f"Customer {i}")
    octane.Customer.create_mapping(name, label="customer", value_regex="name")

count = 0
d = dateutil.parser.isoparse("2021-03-25T00:00:00Z")
while True:
    for i in range(NUM_CUSTOMERS):
        for j in range(NUM_SOURCES):
            octane.Measurement.create(
                meter_name="tot_gauge_meter",
                time=d.isoformat(),
                value=i * count,
                labels={"name": customer_name(i), "source": f"source_{j}"},
            )
    count += 1
    d += timedelta(minutes=1)
    time.sleep(10)


from __future__ import absolute_import, division, print_function

from octane.api_resources.abstract import (
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
)


class Mapping(
    CreateableAPIResource, ListableAPIResource, DeletableAPIResource
):
    OBJECT_NAME = "mapping"

    @classmethod
    def create(cls, sid, **params):
        raise NotImplementedError(
            "Can't create a mapping without a customer name. "
            "Use customer.mappings.create()"
        )

    @classmethod
    def list(cls, sid, **params):
        raise NotImplementedError(
            "Can't list mappings without a customer name. "
            "Use customer.mappings.list()"
        )

    @classmethod
    def retrieve(cls, id, api_key=None, **params):
        raise NotImplementedError("Can't retrieve a mapping.")



#!/usr/bin/env python

import octane
from os import getenv
from coolname import generate_slug
from flask import Flask, request, make_response, send_from_directory

octane.api_key = getenv("OCTANE_API_KEY")

app = Flask("antler-db-cloud-shop")

# Application settings and defaults
port = getenv("APP_PORT", 3000)
bind = getenv("APP_BIND", "127.0.0.1")
octane_redirect_url = getenv("OCTANE_REDIRECT_URL", "https://cloud.getoctane.io")
price_plan_name = getenv("OCTANE_PRICE_PLAN_NAME", "antlerdb")
meter_name_storage = getenv("OCTANE_METER_NAME_STORAGE", "storage")
meter_name_bandwidth = getenv("OCTANE_METER_NAME_BANDWIDTH", "bandwidth")
meter_name_machines = getenv("OCTANE_METER_NAME_MACHINES", "machines")
meter_rate_storage = getenv("env.OCTANE_METER_RATE_STORAGE", 2)
meter_rate_bandwidth = getenv("env.OCTANE_METER_RATE_BANDWIDTH", 5)
meter_rate_machines = getenv("env.OCTANE_METER_RATE_MACHINES", 10)

# The frontend sends us generic resource names,
# which we convert to meter names
resource_meter_map = {
    "storage": meter_name_storage,
    "bandwidth": meter_name_bandwidth,
    "machines": meter_name_machines,
}


@app.route("/")
@app.route("/index.html", methods=["GET"])
def public_index_html():
    return send_from_directory("public", "index.html")


@app.route("/scripts.js", methods=["GET"])
def public_scripts_js():
    return send_from_directory("public", "scripts.js")


@app.route("/styles.css", methods=["GET"])
def public_styles_css():
    return send_from_directory("public", "styles.css")


@app.route("/api/whoami", methods=["GET"])
def api_whoami():
    if request.cookies.get("username"):
        return {
            "code": 200,
            "name": request.cookies.get("username"),
            "url": octane_redirect_url
        }

    # If no cookie, then generate a name and create the customer in Octane
    name = generate_slug(2)
    print(f"[octane] Attempting to create new customer \"{name}\"")

    try:
        octane.Customer.create(
            name=name,
            measurement_mappings=[{
                "label": "customer_name",
                "value_regex": name
            }],
        )
        print(f"[octane] Customer \"{name}\" successfully created")
        print(f"[octane] Attempting to subscribe customer \"{name}\" " +
              f"to price plan \"{price_plan_name}\"")
        try:
            octane.Customer.create_subscription(name, price_plan_name=price_plan_name)
            print(f"[octane] Successfully subscribed customer \"{name}\" " +
                  f"to price plan \"{price_plan_name}\"")
            resp = make_response({
                "code": 201,
                "name": name,
                "url": octane_redirect_url,
            })
            resp.set_cookie("username", name)
            return resp, 201

        except octane.error.APIError as e:
            print("[octane] Error subscribing customer \"{name}\" " +
                  f"to price plan \"{price_plan_name}\"")
            return e.json_body, e.http_status

    except octane.error.APIError as e:
        print("[octane] Error creating customer \"{name}\"")
        return e.json_body, e.http_status


@app.route("/api/resources", methods=["POST"])
def api_resources():
    if not request.cookies.get("username"):
        return {
            "code": 403,
            "message": "No session, please refresh"
        }, 403

    data = request.json
    resource = data["resource"]
    if resource not in resource_meter_map:
        return {
           "code": 400,
           "message": "Invalid resource provided"
        }, 400

    meter_name = resource_meter_map[resource]

    value = data["value"]
    username = request.cookies.get("username")
    print("[octane] Attempting to create measurement " +
           f"for customer \"{username}\"")
    try:
        octane.Measurement.create(
            meter_name=meter_name,
            value=int(value),
            labels={
                "customer_name": username
            }
        )
        print(f"[octane] Measurement for customer \"{username}\" " +
              f"for meter \"{meter_name}\" successfully created")
        return {
            "code": 201,
            "message": "success"
        }, 201

    except octane.error.APIError as e:
        print("[octane] Error creating measurement " +
                      f"for customer \"{username}\" " +
                      f"for meter \"{meter_name}\"")
        return e.json_body, e.http_status


def check_octane_api_key():
    if not octane.api_key:
        raise Exception("Must set OCTANE_API_KEY.")


def check_octane_resource_meter(meter):
    name = meter["name"]
    print(f"[octane] Checking if meter \"{name}\" exists")
    try:
        octane.Meter.retrieve(name)
        print(f"[octane] Meter \"{name}\" already exists")
    except octane.error.APIError as e:
        if e.http_status == 401:
            raise Exception("Unauthorized, please check your OCTANE_API_KEY.")
        print(f"[octane] Meter \"{name}\" does not exist, creating")
        try:
            octane.Meter.create(**meter)  # convert dictionary to keyword args
            print(f"[octane] Meter \"{name}\" successfully created")
        except octane.error.APIError as e:
            print(e.http_body)
            raise Exception("Unable to create meter")


def check_octane_resource_meter_storage():
    check_octane_resource_meter({
        "name": meter_name_storage,
        "display_name": "Storage in gigabytes",
        "meter_type": "COUNTER",
        "unit_name": "gigabyte",
        "is_incremental": True,
        "expected_labels": ["customer_name"]
    })


def check_octane_resource_meter_bandwidth():
    check_octane_resource_meter({
        "name": meter_name_bandwidth,
        "display_name": "Bandwidth in gigabytes",
        "meter_type": "COUNTER",
        "unit_name": "gigabyte",
        "is_incremental": True,
        "expected_labels": ["customer_name"],
    })


def check_octane_resource_meter_machines():
    check_octane_resource_meter({
        "name": meter_name_machines,
        "display_name": "Number of machines",
        "meter_type": "COUNTER",
        "is_incremental": True,
        "expected_labels": ["customer_name"],
    })


def check_octane_resource_price_plan():
    print(f"[octane] Checking if price plan \"{price_plan_name}\" exists")
    try:
        octane.PricePlan.retrieve(price_plan_name)
        print(f"[octane] Price plan \"{price_plan_name}\" already exists")
    except octane.error.APIError as e:
        if e.http_status == 401:
            raise Exception("Unauthorized, please check your OCTANE_API_KEY.")
        print(f"[octane] Price plan \"{price_plan_name}\" does not exist, creating")
        try:
            octane.PricePlan.create(
                name=price_plan_name,
                period="month",
                metered_components=[
                    {
                        "meter_name": meter_name_storage,
                        "price_scheme": {
                            "prices": [{
                                "price": int(meter_rate_storage) * 100
                            }],
                            "scheme_type": "FLAT",
                            "unit_name": "gigabyte"
                        }
                    },
                    {
                        "meter_name": meter_name_bandwidth,
                        "price_scheme": {
                            "prices": [{
                                "price": int(meter_rate_bandwidth) * 100
                            }],
                            "scheme_type": "FLAT",
                            "unit_name": "gigabyte"
                        }
                    },
                    {
                        "meter_name": meter_name_machines,
                        "price_scheme": {
                            "prices": [{
                                "price": int(meter_rate_machines) * 100
                            }],
                            "scheme_type": "FLAT"
                        }
                    }
                ]
            )
            print(f"[octane] Price plan \"{price_plan_name}\" successfully created")
        except octane.error.APIError as e:
            print(e.http_body)
            raise Exception("Unable to create price plan")


if __name__ == "__main__":
    check_octane_api_key()
    check_octane_resource_meter_storage()
    check_octane_resource_meter_bandwidth()
    check_octane_resource_meter_machines()
    check_octane_resource_price_plan()
    print(f"[server] Listening at http://{bind}:{port}/")
    app.run(host=bind, port=port)



from __future__ import absolute_import, division, print_function
from octane.octane_object import OctaneObject

from octane import api_requestor, util
from octane.api_resources.abstract.api_resource import APIResource


class ListableAPIResource(APIResource):
    @classmethod
    def auto_paging_iter(cls, *args, **params):
        return cls.list(*args, **params).auto_paging_iter()

    @classmethod
    def list(cls, api_key=None, octane_version=None, **params):
        requestor = api_requestor.APIRequestor(
            api_key,
            api_base=cls.api_base(),
            api_version=octane_version,
        )
        url = cls.class_url()
        response, api_key = requestor.request("get", url, params)
        octane_object = util.convert_to_octane_object(
            response, api_key, octane_version
        )

        def _add_params(obj, params):
            if isinstance(obj, OctaneObject):
                obj._retreieve_params = params
                return

            if isinstance(obj, list):
                for e in obj:
                    _add_params(e, params)

        _add_params(octane_object, params)

        return octane_object



from __future__ import absolute_import, division, print_function

from octane.api_resources.abstract import (
    CreateableAPIResource,
)


class PaymentGatewayCredential(
    CreateableAPIResource
):
    OBJECT_NAME = "payment_gateway_credential"

    @classmethod
    def retrieve(cls, id, api_key=None, **params):
        raise NotImplementedError("Can't retrieve a payment_gateway_credential.")



from __future__ import absolute_import, division, print_function

import octane

BASE_URL = "/measurements"
TEST_RESOURCE_ID = "measurement_123"


class TestMeasurement(object):
    def test_is_creatable(self, request_mock):
        resource = octane.Measurement.create()
        request_mock.assert_requested("post", BASE_URL)



from __future__ import absolute_import, division, print_function
from octane.api_resources.abstract.custom_method import custom_method
from octane.api_resources.abstract import nested_resource_class_methods

from octane import util
from octane.api_resources.abstract import CreateableAPIResource
from octane.api_resources.abstract import DeletableAPIResource
from octane.api_resources.abstract import ListableAPIResource
from octane.api_resources.abstract import UpdateableAPIResource


@nested_resource_class_methods(
    "subscription", operations=["create"]
)
@nested_resource_class_methods(
    "mapping", operations=["create", "list", "delete"]
)
@nested_resource_class_methods(
    "payment_gateway_credential", operations=["create"]
)
@custom_method(
    "revenue", "get"
)
@custom_method(
    "usage", "get"
)
class Customer(
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    UpdateableAPIResource,
):
    OBJECT_NAME = "customer"

    def revenue(self, idempotency_key=None, **params):
        url = self.instance_url() + "/revenue"
        headers = util.populate_headers(idempotency_key)
        self.refresh_from(self.request("get", url, params, headers))
        return self

    def usage(self, idempotency_key=None, **params):
        url = self.instance_url() + "/usage"
        headers = util.populate_headers(idempotency_key)
        self.refresh_from(self.request("get", url, params, headers))
        return self


from __future__ import absolute_import, division, print_function

# flake8: noqa

from octane.api_resources.error_object import ErrorObject
from octane.api_resources.list_object import ListObject

from octane.api_resources.customer import Customer
from octane.api_resources.mapping import Mapping
from octane.api_resources.measurement import Measurement
from octane.api_resources.meter import Meter
from octane.api_resources.price_plan import PricePlan
from octane.api_resources.price_list import PriceList
from octane.api_resources.payment_gateway_credential import PaymentGatewayCredential
from octane.api_resources.subscription import Subscription
